@@ -2,7 +2,7 @@
  *            qsf-backend.c
  *
  *  Sat Jan  1 15:07:14 2005
- *  Copyright  2005  Neil Williams
+ *  Copyright  2005, 2006  Neil Williams
  *  linux@codehelp.co.uk
  ****************************************************************************/
 /*
@@ -14,7 +14,7 @@
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
@@ -23,25 +23,32 @@
 
 #define _GNU_SOURCE
 
+#include "config.h"
+#include <errno.h>
+#include <sys/stat.h>
+#include <glib.h>
+#include <libxml/xmlmemory.h>
+#include <libxml/tree.h>
+#include <libxml/parser.h>
+#include <libxml/xmlschemas.h>
 #include "qof.h"
 #include "qof-backend-qsf.h"
 #include "qsf-xml.h"
 #include "qsf-dir.h"
-#include <errno.h>
-#include <sys/stat.h>
 
 #define QSF_TYPE_BINARY "binary"
 #define QSF_TYPE_GLIST  "glist"
 #define QSF_TYPE_FRAME  "frame"
 
 static QofLogModule log_module = QOF_MOD_QSF;
+
 static void qsf_object_commitCB(gpointer key, gpointer value, gpointer data);
 
 struct QSFBackend_s
 {
 	QofBackend be;
 	qsf_param *params;
-	char *fullpath;
+	gchar *fullpath;
 };
 
 typedef struct QSFBackend_s QSFBackend;
@@ -53,10 +60,15 @@ static void option_cb (QofBackendOption *option, gpointer data)
 	params = (qsf_param*)data;
 	g_return_if_fail(params);
 	if(0 == safe_strcmp(QSF_COMPRESS, option->option_name)) {
-		params->use_gz_level = GPOINTER_TO_INT(option->value);
+		params->use_gz_level = (*(gint64*)option->value);
+		PINFO (" compression=%" G_GINT64_FORMAT,params->use_gz_level);
 	}
 	if (0 == safe_strcmp(QSF_MAP_FILES, option->option_name)) {
-		params->map_files = g_list_copy(option->value);
+		params->map_files = g_list_copy((GList*)option->value);
+	}
+	if (0 == safe_strcmp(QSF_ENCODING, option->option_name)) {
+		params->encoding = g_strdup(option->value);
+		PINFO (" encoding=%s", params->encoding);
 	}
 }
 
@@ -66,10 +78,12 @@ qsf_load_config(QofBackend *be, KvpFrame *config)
 	QSFBackend *qsf_be;
 	qsf_param  *params;
 
+	ENTER (" ");
 	qsf_be = (QSFBackend*)be;
 	g_return_if_fail(qsf_be->params);
 	params = qsf_be->params;
 	qof_backend_option_foreach(config, option_cb, params);
+	LEAVE (" ");
 }
 
 static KvpFrame*
@@ -80,6 +94,7 @@ qsf_get_config(QofBackend *be)
 	qsf_param *params;
 
 	if(!be) { return NULL; }
+	ENTER (" ");
 	qsf_be = (QSFBackend*)be;
 	g_return_val_if_fail(qsf_be->params, NULL);
 	params = qsf_be->params;
@@ -90,7 +105,8 @@ qsf_get_config(QofBackend *be)
 	option->tooltip = _("QOF can compress QSF XML files using gzip. "
 		"Note that compression is not used when outputting to STDOUT.");
 	option->type = KVP_TYPE_GINT64;
-	option->value = GINT_TO_POINTER(params->use_gz_level);
+	/* GINT_TO_POINTER can only be used for 32bit values. */
+	option->value = (gpointer)&params->use_gz_level;
 	qof_backend_prepare_option(be, option);
 	g_free(option);
 	option = g_new0(QofBackendOption, 1);
@@ -99,16 +115,29 @@ qsf_get_config(QofBackend *be)
 	option->tooltip = _("QOF can convert objects within QSF XML files "
 		"using a map of the changes required.");
 	option->type = KVP_TYPE_GLIST;
-	option->value = params->map_files;
+	option->value = (gpointer)params->map_files;
 	qof_backend_prepare_option(be, option);
 	g_free(option);
+	option = g_new0(QofBackendOption, 1);
+	option->option_name = QSF_ENCODING;
+	option->description = _("Encoding string to use when writing the XML file.");
+	option->tooltip = _("QSF defaults to UTF-8. Other encodings are supported by "
+			"passing the encoding string in this option.");
+	option->type = KVP_TYPE_STRING;
+	option->value = (gpointer)params->encoding;
+	qof_backend_prepare_option(be, option);
+	g_free(option);
+	LEAVE (" ");
 	return qof_backend_complete_frame(be);
 }
 
 GList**
 qsf_map_prepare_list(GList **maps)
 {
+	/* Add new map filenames here. */
+	/** \todo Automate this once map support is stable */
 	*maps = g_list_prepend(*maps, "pilot-qsf-GnuCashInvoice.xml");
+	*maps = g_list_prepend(*maps, "pilot-qsf-gncCustomer.xml");
 	return maps;
 }
 
@@ -135,6 +164,8 @@ qsf_param_init(qsf_param *params)
 	params->full_kvp_path = NULL;
 	params->map_ns = NULL;
 	params->map_files = NULL;
+	params->map_path = NULL;
+	params->encoding = "UTF-8";
 	params->qsf_object_list = NULL;
 	params->qsf_parameter_hash = g_hash_table_new(g_str_hash, g_str_equal);
 	params->qsf_default_hash = g_hash_table_new(g_str_hash, g_str_equal);
@@ -169,7 +200,7 @@ qsf_param_init(qsf_param *params)
 }
 
 static gboolean 
-qsf_determine_file_type(const char *path)
+qsf_determine_file_type(const gchar *path)
 {
 	struct stat sbuf;
 
@@ -179,7 +210,7 @@ qsf_determine_file_type(const char *path)
 	if (sbuf.st_size == 0)       { return TRUE; }
 	if(is_our_qsf_object(path))  { return TRUE; }
 	else if(is_qsf_object(path)) { return TRUE; }
-    else if(is_qsf_map(path))    { return TRUE; }
+	else if(is_qsf_map(path))    { return TRUE; }
 	return FALSE;
 }
 
@@ -187,11 +218,11 @@ qsf_determine_file_type(const char *path)
 Just strip the file: from the start of the book_path URL. Locks are not implemented.
 */
 static void
-qsf_session_begin(QofBackend *be, QofSession *session, const char *book_path,
+qsf_session_begin(QofBackend *be, QofSession *session, const gchar *book_path,
                    gboolean ignore_lock, gboolean create_if_nonexistent)
 {
 	QSFBackend *qsf_be;
-	char *p, *path;
+	gchar *p, *path;
 
 	PINFO (" ignore_lock=%d create_if_nonexistent=%d", ignore_lock, create_if_nonexistent);
 	g_return_if_fail(be != NULL);
@@ -207,8 +238,8 @@ qsf_session_begin(QofBackend *be, QofSession *session, const char *book_path,
 	p = strchr (book_path, ':');
 	if (p) {
 		path = g_strdup (book_path);
-		if (!g_strncasecmp(path, "file:", 5)) {
-			p = g_new(char, strlen(path) - 5 + 1);
+		if (!g_ascii_strncasecmp(path, "file:", 5)) {
+			p = g_new(gchar, strlen(path) - 5 + 1);
 			strcpy(p, path + 5);
 		}
 		qsf_be->fullpath = g_strdup(p);
@@ -219,10 +250,10 @@ qsf_session_begin(QofBackend *be, QofSession *session, const char *book_path,
 	}
 	if(create_if_nonexistent)
 	{
-        FILE *f;
-
-        f = fopen(qsf_be->fullpath, "a+");
-        if(f) {fclose(f); }
+		FILE *f;
+		
+		f = fopen(qsf_be->fullpath, "a+");
+		if(f) {fclose(f); }
 		else
 		{
 			qof_backend_set_error(be, ERR_BACKEND_READONLY);
@@ -334,14 +365,17 @@ qsfdoc_to_qofbook(xmlDocPtr doc, qsf_param *params)
 	while(object_list != NULL)
 	{
 		params->object_set = object_list->data;
+		object_list = g_list_next(object_list);
 		params->qsf_parameter_hash = params->object_set->parameters;
-		inst = (QofInstance*)qof_object_new_instance(params->object_set->object_type, book);
+		if(!qof_class_is_registered(params->object_set->object_type)) { continue; }
+		inst = (QofInstance*)qof_object_new_instance(params->object_set->object_type, 
+			book);
 		g_return_val_if_fail(inst != NULL, FALSE);
 		params->qsf_ent = &inst->entity;
 		qof_begin_edit(inst);
-		g_hash_table_foreach(params->qsf_parameter_hash, qsf_object_commitCB, params);
+		g_hash_table_foreach(params->qsf_parameter_hash, qsf_object_commitCB, 
+			params);
 		qof_commit_edit(inst);
-		object_list = g_list_next(object_list);
 	}
 	qof_object_foreach_type(insert_ref_cb, params);
 	qof_book_set_data(book, ENTITYREFERENCE, params->referenceList);
@@ -351,22 +385,19 @@ qsfdoc_to_qofbook(xmlDocPtr doc, qsf_param *params)
 /* QofBackend routine to load from file - needs a map.
 */
 static gboolean
-load_qsf_object(QofBook *book, const char *fullpath, qsf_param *params)
+load_qsf_object(QofBook *book, const gchar *fullpath, qsf_param *params)
 {
 	xmlNodePtr qsf_root, map_root;
 	xmlDocPtr mapDoc, foreign_doc;
 	gchar *map_path, *map_file;
-	GList *maps;
 
-	maps = params->map_files;
-	map_path = NULL;
+	map_file = params->map_path;
 	mapDoc = NULL;
-	if(!maps) {
+	/* use selected map */
+	if(!map_file) {
 		qof_backend_set_error(params->be, ERR_QSF_NO_MAP);
-		return FALSE;
+		return FALSE; 
 	}
-	/* use default maps */
-	map_file = g_strdup(maps->data);
 	foreign_doc = xmlParseFile(fullpath);
 	if (foreign_doc == NULL) {
 		qof_backend_set_error(params->be, ERR_FILEIO_PARSE_ERROR);
@@ -390,12 +421,11 @@ load_qsf_object(QofBook *book, const char *fullpath, qsf_param *params)
 	params->map_ns = map_root->ns;
 	params->input_doc = qsf_object_convert(mapDoc, qsf_root, params);
 	qsfdoc_to_qofbook(params->input_doc, params);
-	g_free(map_file);
 	return TRUE;
 }
 
 static gboolean
-load_our_qsf_object(QofBook *book, const char *fullpath, qsf_param *params)
+load_our_qsf_object(QofBook *book, const gchar *fullpath, qsf_param *params)
 {
 	xmlNodePtr qsf_root;
 
@@ -413,26 +443,28 @@ load_our_qsf_object(QofBook *book, const char *fullpath, qsf_param *params)
 /* Determine the type of QSF and load it into the QofBook
 
 - is_our_qsf_object, OUR_QSF_OBJ, QSF object file using only QOF objects known
-	to the calling process.	No map is required.
+	to the calling process. No map is required.
 - is_qsf_object, IS_QSF_OBJ, QSF object file that may or may not have a QSF map
 	to convert external objects. This temporary type will be set to HAVE_QSF_MAP 
 	if a suitable map exists, or an error value returned: ERR_QSF_NO_MAP, 
 	ERR_QSF_BAD_MAP or ERR_QSF_WRONG_MAP. This allows the calling process to inform 
 	the user that the QSF itself is valid but a suitable map cannot be found.
 - is_qsf_map, IS_QSF_MAP, QSF map file. In the backend, this generates 
-	ERR_QSF_MAP_NOT_OBJ but	it can be used internally when processing maps to 
+	ERR_QSF_MAP_NOT_OBJ but it can be used internally when processing maps to 
 	match a QSF object.
 
 returns NULL on error, otherwise a pointer to the QofBook. Use
-	the qof_book_merge API to merge the new data into the current
-	QofBook. 
+the qof_book_merge API to merge the new data into the current
+QofBook. 
 */
 static void
 qsf_file_type(QofBackend *be, QofBook *book)
 {
 	QSFBackend *qsf_be;
+	QofBackendError err;
 	qsf_param *params;
-	char *path;
+	FILE *f;
+	gchar *path;
 	gboolean result;
 
 	g_return_if_fail(be != NULL);
@@ -444,6 +476,9 @@ qsf_file_type(QofBackend *be, QofBook *book)
 	params = qsf_be->params;
 	params->book = book;
 	path = g_strdup(qsf_be->fullpath);
+	f = fopen(path, "r");
+	if(!f) { qof_backend_set_error(be, ERR_FILEIO_READ_ERROR); }
+	fclose(f);
 	params->filepath = g_strdup(path);
 	qof_backend_get_error(be);
 	result = is_our_qsf_object_be(params);
@@ -457,8 +492,18 @@ qsf_file_type(QofBackend *be, QofBook *book)
 		params->file_type = IS_QSF_OBJ;
 		result = load_qsf_object(book, path, params);
 		if(!result) { qof_backend_set_error(be, ERR_FILEIO_PARSE_ERROR); }
+		return;
 	}
-	if(result == FALSE) {
+	err = qof_backend_get_error(be);
+	if(err == ERR_QSF_WRONG_MAP)
+	{
+		/* usable QSF object but no map available */
+		params->file_type = IS_QSF_OBJ;
+		result = TRUE;
+	}
+	/* pop the error back on the stack. */
+	qof_backend_set_error(params->be, err);
+	if(result == FALSE){
 		if(is_qsf_map_be(params)) {
 		params->file_type = IS_QSF_MAP;
 		qof_backend_set_error(be, ERR_QSF_MAP_NOT_OBJ);
@@ -551,7 +596,7 @@ kvp_value_to_qof_type_helper(KvpValueType n)
 
 
 static void
-qsf_from_kvp_helper(const char *path, KvpValue *content, gpointer data)
+qsf_from_kvp_helper(const gchar *path, KvpValue *content, gpointer data)
 {
 	qsf_param *params;
 	QofParam *qof_param;
@@ -579,7 +624,8 @@ qsf_from_kvp_helper(const char *path, KvpValue *content, gpointer data)
 			node = xmlAddChild(params->output_node, xmlNewNode(params->qsf_ns,
 				BAD_CAST qof_param->param_type));
 			xmlNodeAddContent(node, BAD_CAST kvp_value_to_bare_string(content));
-			xmlNewProp(node, BAD_CAST QSF_OBJECT_TYPE, BAD_CAST qof_param->param_name);
+			xmlNewProp(node, BAD_CAST QSF_OBJECT_TYPE, BAD_CAST 
+				qof_param->param_name);
 			full_path = g_strconcat(params->full_kvp_path, "/", path, NULL);
 			xmlNewProp(node, BAD_CAST QSF_OBJECT_KVP, BAD_CAST full_path);
 			xmlNewProp(node, BAD_CAST QSF_OBJECT_VALUE, 
@@ -621,9 +667,9 @@ qsf_from_coll_cb (QofEntity *ent, gpointer user_data)
 	params = (qsf_param*)user_data;
 	if(!ent || !params) { return; }
 	qof_param = params->qof_param;
+	guid_to_string_buff(qof_entity_get_guid(ent), qsf_guid);
 	node = xmlAddChild(params->output_node, xmlNewNode(params->qsf_ns,
 		BAD_CAST qof_param->param_type));
-	guid_to_string_buff(qof_entity_get_guid(ent), qsf_guid);
 	xmlNodeAddContent(node, BAD_CAST qsf_guid);
 	xmlNewProp(node, BAD_CAST QSF_OBJECT_TYPE, BAD_CAST qof_param->param_name);
 }
@@ -674,6 +720,7 @@ reference_list_lookup(gpointer data, gpointer user_data)
 	QofParam *ref_param;
 	QofEntityReference *reference, *starter;
 	qsf_param  *params;
+	const GUID *guid;
 	xmlNodePtr node, object_node;
 	xmlNsPtr ns;
 	GList *copy_list;
@@ -704,6 +751,18 @@ reference_list_lookup(gpointer data, gpointer user_data)
 		xmlNewProp(node, BAD_CAST QSF_OBJECT_TYPE, BAD_CAST ref_name);
 		g_free(ref_name);
 	}
+	else {
+		ent = (QofEntity*)ref_param->param_getfcn(ent, ref_param);
+		if(!ent) { return; }
+		if((0 == safe_strcmp(ref_param->param_type, QOF_TYPE_COLLECT)) ||
+			(0 == safe_strcmp(ref_param->param_type, QOF_TYPE_CHOICE)))
+		{ return; }
+		node = xmlAddChild(object_node, xmlNewNode(ns, BAD_CAST QOF_TYPE_GUID));
+		guid = qof_entity_get_guid(ent);
+		guid_to_string_buff(guid, qsf_guid);
+		xmlNodeAddContent(node, BAD_CAST qsf_guid);
+		xmlNewProp(node, BAD_CAST QSF_OBJECT_TYPE, BAD_CAST ref_param->param_name);
+	}
 }
 
 /*=====================================
@@ -719,15 +778,14 @@ qsf_entity_foreach(QofEntity *ent, gpointer data)
 	xmlNodePtr node, object_node;
 	xmlNsPtr   ns;
 	gchar      *string_buffer;
-	GString    *buffer;
 	QofParam   *qof_param;
 	QofEntity  *choice_ent;
 	KvpFrame   *qsf_kvp;
 	QofCollection *qsf_coll;
-	int        param_count;
+	gint        param_count;
 	gboolean   own_guid;
 	const GUID *cm_guid;
-	char       cm_sa[GUID_ENCODING_LENGTH + 1];
+	gchar       cm_sa[GUID_ENCODING_LENGTH + 1];
 
 	g_return_if_fail(data != NULL);
 	params = (qsf_param*)data;
@@ -739,9 +797,9 @@ qsf_entity_foreach(QofEntity *ent, gpointer data)
 	object_node = xmlNewChild(params->book_node, params->qsf_ns,
 		BAD_CAST QSF_OBJECT_TAG, NULL);
 	xmlNewProp(object_node, BAD_CAST QSF_OBJECT_TYPE, BAD_CAST ent->e_type);
-	buffer = g_string_new(" ");
-	g_string_printf(buffer, "%i", param_count);
-	xmlNewProp(object_node, BAD_CAST QSF_OBJECT_COUNT, BAD_CAST buffer->str);
+	string_buffer = g_strdup_printf("%i", param_count);
+	xmlNewProp(object_node, BAD_CAST QSF_OBJECT_COUNT, BAD_CAST string_buffer);
+	g_free(string_buffer);
 	param_list = g_slist_copy(params->qsf_sequence);
 	while(param_list != NULL) {
 		qof_param = (QofParam*)param_list->data;
@@ -751,11 +809,14 @@ qsf_entity_foreach(QofEntity *ent, gpointer data)
 			if(!own_guid)
 			{
 				cm_guid = qof_entity_get_guid(ent);
-				node = xmlAddChild(object_node, xmlNewNode(ns, BAD_CAST QOF_TYPE_GUID));
+				node = xmlAddChild(object_node, xmlNewNode(ns, BAD_CAST 
+					QOF_TYPE_GUID));
 				guid_to_string_buff(cm_guid, cm_sa);
 				string_buffer = g_strdup(cm_sa);
 				xmlNodeAddContent(node, BAD_CAST string_buffer);
-				xmlNewProp(node, BAD_CAST QSF_OBJECT_TYPE , BAD_CAST QOF_PARAM_GUID);
+				xmlNewProp(node, BAD_CAST QSF_OBJECT_TYPE , BAD_CAST 
+					QOF_PARAM_GUID);
+				g_free(string_buffer);
 				own_guid = TRUE;
 			}
 			params->qsf_ent = ent;
@@ -768,27 +829,34 @@ qsf_entity_foreach(QofEntity *ent, gpointer data)
 		if(0 == safe_strcmp(qof_param->param_type, QOF_TYPE_COLLECT))
 		{
 			qsf_coll = qof_param->param_getfcn(ent, qof_param);
-			params->qof_param = qof_param;
-			params->output_node = object_node;
-			qof_collection_foreach(qsf_coll, qsf_from_coll_cb, params);
+			if(qsf_coll) {
+				params->qof_param = qof_param;
+				params->output_node = object_node;
+				if(qof_collection_count(qsf_coll) > 0) {
+					qof_collection_foreach(qsf_coll, qsf_from_coll_cb, params);
+				}
+			}
+			param_list = g_slist_next(param_list);
+			continue;
 		}
 		if(0 == safe_strcmp(qof_param->param_type, QOF_TYPE_CHOICE))
 		{
 			/** \todo use the reference list here. */
-			/* repeats due to use of reference list for GUID and CHOICE */
-			/* fix to allow COLLECT too */
 			choice_ent = (QofEntity*)qof_param->param_getfcn(ent, qof_param);
 			if(!choice_ent) {
 				param_list = g_slist_next(param_list);
 				continue;
 			}
-			node = xmlAddChild(object_node, xmlNewNode(ns, BAD_CAST qof_param->param_type));
+			node = xmlAddChild(object_node, xmlNewNode(ns, BAD_CAST 
+				qof_param->param_type));
 			cm_guid = qof_entity_get_guid(choice_ent);
 			guid_to_string_buff(cm_guid, cm_sa);
 			string_buffer = g_strdup(cm_sa);
 			xmlNodeAddContent(node, BAD_CAST string_buffer);
-			xmlNewProp(node, BAD_CAST QSF_OBJECT_TYPE, BAD_CAST qof_param->param_name);
+			xmlNewProp(node, BAD_CAST QSF_OBJECT_TYPE, BAD_CAST 
+				qof_param->param_name);
 			xmlNewProp(node, BAD_CAST "name", BAD_CAST choice_ent->e_type);
+			g_free(string_buffer);
 			param_list = g_slist_next(param_list);
 			continue;
 		}
@@ -805,12 +873,17 @@ qsf_entity_foreach(QofEntity *ent, gpointer data)
 			for( supported = g_slist_copy(params->supported_types);
 				supported != NULL; supported = g_slist_next(supported))
 			{
-				if(0 == safe_strcmp((const char*)supported->data, (const char*)qof_param->param_type))
+				if(0 == safe_strcmp((const gchar*)supported->data, 
+					(const gchar*)qof_param->param_type))
 				{
-					node = xmlAddChild(object_node, xmlNewNode(ns, BAD_CAST qof_param->param_type));
-					string_buffer = g_strdup(qof_book_merge_param_as_string(qof_param, ent));
+					node = xmlAddChild(object_node, 
+						xmlNewNode(ns, BAD_CAST qof_param->param_type));
+					string_buffer = g_strdup(qof_book_merge_param_as_string(qof_param, 
+						ent));
 					xmlNodeAddContent(node, BAD_CAST string_buffer);
-					xmlNewProp(node, BAD_CAST QSF_OBJECT_TYPE, BAD_CAST qof_param->param_name);
+					xmlNewProp(node, BAD_CAST QSF_OBJECT_TYPE, BAD_CAST 
+						qof_param->param_name);
+					g_free(string_buffer);
 				}
 			}
 		}
@@ -884,12 +957,15 @@ write_qsf_from_book(const char *path, QofBook *book, qsf_param *params)
 	be = qof_book_get_backend(book);
 	qsf_doc = qofbook_to_qsf(book, params);
 	write_result = 0;
+	PINFO (" use_gz_level=%" G_GINT64_FORMAT " encoding=%s",
+		params->use_gz_level, params->encoding);
 	if((params->use_gz_level > 0) && (params->use_gz_level <= 9)) 
 	{
 		xmlSetDocCompressMode(qsf_doc, params->use_gz_level); 
 	}
-	g_return_if_fail(qsf_is_valid(QSF_SCHEMA_DIR, QSF_OBJECT_SCHEMA, qsf_doc) == TRUE);
-	write_result = xmlSaveFormatFileEnc(path, qsf_doc, "UTF-8", 1);
+	g_return_if_fail(
+		qsf_is_valid(QSF_SCHEMA_DIR, QSF_OBJECT_SCHEMA, qsf_doc) == TRUE);
+	write_result = xmlSaveFormatFileEnc(path, qsf_doc, params->encoding, 1);
 	if(write_result < 0) 
 	{
 		qof_backend_set_error(be, ERR_FILEIO_WRITE_ERROR);
@@ -904,8 +980,11 @@ write_qsf_to_stdout(QofBook *book, qsf_param *params)
 	xmlDocPtr qsf_doc;
 
 	qsf_doc = qofbook_to_qsf(book, params);
-	g_return_if_fail(qsf_is_valid(QSF_SCHEMA_DIR, QSF_OBJECT_SCHEMA, qsf_doc) == TRUE);
-	xmlSaveFormatFileEnc("-", qsf_doc, "UTF-8", 1);
+	g_return_if_fail(
+		qsf_is_valid(QSF_SCHEMA_DIR, QSF_OBJECT_SCHEMA, qsf_doc) == TRUE);
+	PINFO (" use_gz_level=%" G_GINT64_FORMAT " encoding=%s", 
+		params->use_gz_level, params->encoding);
+	xmlSaveFormatFileEnc("-", qsf_doc, params->encoding, 1);
 	fprintf(stdout, "\n");
 	xmlFreeDoc(qsf_doc);
 }
@@ -930,9 +1009,9 @@ qsf_write_file(QofBackend *be, QofBook *book)
 }
 
 KvpValue*
-string_to_kvp_value(const char *content, KvpValueType type)
+string_to_kvp_value(const gchar *content, KvpValueType type)
 {
-	char        *tail;
+	gchar        *tail;
 	gint64      cm_i64;
 	double      cm_double;
 	gnc_numeric cm_numeric;
@@ -941,49 +1020,62 @@ string_to_kvp_value(const char *content, KvpValueType type)
 	time_t      kvp_time_t;
 	Timespec    cm_date;
 
-	switch(type) {
-	  case KVP_TYPE_GINT64:
-		errno = 0;
-		cm_i64 = strtoll(content, &tail, 0);
-		if(errno == 0) {
-			return kvp_value_new_gint64(cm_i64);
-		}
-		break;
-	  case KVP_TYPE_DOUBLE:
-  		errno = 0;
-		cm_double = strtod(content, &tail);
-		if(errno == 0) {
-			return kvp_value_new_double(cm_double);
-		}
-		break;
-	  case KVP_TYPE_NUMERIC:
-		string_to_gnc_numeric(content, &cm_numeric);
-		return kvp_value_new_gnc_numeric(cm_numeric);
-		break;
-	  case KVP_TYPE_STRING:
-		return kvp_value_new_string(content);
-		break;
-	  case KVP_TYPE_GUID:
-		cm_guid = g_new(GUID, 1);
-		if(TRUE == string_to_guid(content, cm_guid))
+	switch(type)
+	{
+		case KVP_TYPE_GINT64:
 		{
-			return kvp_value_new_guid(cm_guid);
+			errno = 0;
+			cm_i64 = strtoll(content, &tail, 0);
+			if(errno == 0) {
+				return kvp_value_new_gint64(cm_i64);
+			}
+			break;
 		}
-		break;
-	  case KVP_TYPE_TIMESPEC:
-		strptime(content, QSF_XSD_TIME, &kvp_time);
-		kvp_time_t = mktime(&kvp_time);
-		timespecFromTime_t(&cm_date, kvp_time_t);
-		return kvp_value_new_timespec(cm_date);
-		break;
-	  case KVP_TYPE_BINARY:
+		case KVP_TYPE_DOUBLE:
+		{
+			errno = 0;
+			cm_double = strtod(content, &tail);
+			if(errno == 0) {
+				return kvp_value_new_double(cm_double);
+			}
+			break;
+		}
+		case KVP_TYPE_NUMERIC:
+		{
+			string_to_gnc_numeric(content, &cm_numeric);
+			return kvp_value_new_gnc_numeric(cm_numeric);
+			break;
+		}
+		case KVP_TYPE_STRING:
+		{
+			return kvp_value_new_string(content);
+			break;
+		}
+		case KVP_TYPE_GUID:
+		{
+			cm_guid = g_new(GUID, 1);
+			if(TRUE == string_to_guid(content, cm_guid))
+			{
+				return kvp_value_new_guid(cm_guid);
+			}
+			break;
+		}
+		case KVP_TYPE_TIMESPEC:
+		{
+			strptime(content, QSF_XSD_TIME, &kvp_time);
+			kvp_time_t = mktime(&kvp_time);
+			timespecFromTime_t(&cm_date, kvp_time_t);
+			return kvp_value_new_timespec(cm_date);
+			break;
+		}
+		case KVP_TYPE_BINARY:
 //		return kvp_value_new_binary(value->value.binary.data,
 //									value->value.binary.datasize);
 		break;
-	  case KVP_TYPE_GLIST:
+	case KVP_TYPE_GLIST:
 //		return kvp_value_new_glist(value->value.list);
 		break;
-	  case KVP_TYPE_FRAME:
+	case KVP_TYPE_FRAME:
 //		return kvp_value_new_frame(value->value.frame);
 		break;
 	}
@@ -1006,7 +1098,7 @@ qsf_object_commitCB(gpointer key, gpointer value, gpointer data)
 	QofIdType          obj_type, reference_type;
 	struct tm          qsf_time;
 	time_t             qsf_time_t;
-	char               *tail;
+	gchar              *tail;
 	/* cm_ prefix used for variables that hold the data to commit */
 	gnc_numeric    cm_numeric;
 	double         cm_double;
@@ -1014,49 +1106,54 @@ qsf_object_commitCB(gpointer key, gpointer value, gpointer data)
 	gint32         cm_i32;
 	gint64         cm_i64;
 	Timespec       cm_date;
-	char           cm_char,    (*char_getter)  (xmlNodePtr);
+	gchar          cm_char,    (*char_getter)  (xmlNodePtr);
 	GUID           *cm_guid;
 	KvpFrame       *cm_kvp;
 	KvpValue       *cm_value;
 	KvpValueType   cm_type;
 	QofSetterFunc  cm_setter;
 	const QofParam *cm_param;
-	void (*string_setter)    (QofEntity*, const char*);
+	void (*string_setter)    (QofEntity*, const gchar*);
 	void (*date_setter)      (QofEntity*, Timespec);
 	void (*numeric_setter)   (QofEntity*, gnc_numeric);
 	void (*double_setter)    (QofEntity*, double);
 	void (*boolean_setter)   (QofEntity*, gboolean);
 	void (*i32_setter)       (QofEntity*, gint32);
 	void (*i64_setter)       (QofEntity*, gint64);
-	void (*char_setter)      (QofEntity*, char);
+	void (*char_setter)      (QofEntity*, gchar);
 
 	g_return_if_fail(data && value && key);
 	params = (qsf_param*)data;
 	node = (xmlNodePtr)value;
-	parameter_name = (const char*)key;
-	qof_type = (char*)node->name;
+	parameter_name = (const gchar*)key;
+	qof_type = (gchar*)node->name;
 	qsf_ent = params->qsf_ent;
 	targetBook = params->book;
 	memset (&qsf_time, '\0', sizeof(qsf_time));
 	cm_date.tv_nsec = 0;
 	cm_date.tv_sec = 0;
-	obj_type = (char*)xmlGetProp(node->parent, BAD_CAST QSF_OBJECT_TYPE);
+	obj_type = (gchar*)xmlGetProp(node->parent, BAD_CAST QSF_OBJECT_TYPE);
 	if(0 == safe_strcasecmp(obj_type, parameter_name)) { return; }
 	cm_setter = qof_class_get_parameter_setter(obj_type, parameter_name);
 	cm_param = qof_class_get_parameter(obj_type, parameter_name);
 	object_set = params->object_set;
 	if(safe_strcmp(qof_type, QOF_TYPE_STRING) == 0)  {
-		string_setter = (void(*)(QofEntity*, const char*))cm_setter;
-		if(string_setter != NULL) { string_setter(qsf_ent, (char*)xmlNodeGetContent(node)); }
+		string_setter = (void(*)(QofEntity*, const gchar*))cm_setter;
+		if(string_setter != NULL) { string_setter(qsf_ent, 
+			(gchar*)xmlNodeGetContent(node)); }
 	}
 	if(safe_strcmp(qof_type, QOF_TYPE_DATE) == 0) {
 		date_setter = (void(*)(QofEntity*, Timespec))cm_setter;
 		timechk = NULL;
-		timechk = strptime((char*)xmlNodeGetContent(node), QSF_XSD_TIME, &qsf_time);
+		timechk = strptime((char*)xmlNodeGetContent(node), QSF_XSD_TIME, 
+			&qsf_time);
 		g_return_if_fail(timechk != NULL);
 		qsf_time_t = mktime(&qsf_time);
-		timespecFromTime_t(&cm_date, qsf_time_t);
-		if(date_setter != NULL) { date_setter(qsf_ent, cm_date); }
+		if(qsf_time_t != -3600)
+		{
+			timespecFromTime_t(&cm_date, qsf_time_t);
+			if(date_setter != NULL) { date_setter(qsf_ent, cm_date); }
+		}
 	}
 	if((safe_strcmp(qof_type, QOF_TYPE_NUMERIC) == 0)  ||
 	(safe_strcmp(qof_type, QOF_TYPE_DEBCRED) == 0)) {
@@ -1081,7 +1178,8 @@ qsf_object_commitCB(gpointer key, gpointer value, gpointer data)
 		else {
 			reference = qof_entity_get_reference_from(qsf_ent, cm_param);
 			if(reference) {
-				params->referenceList = g_list_append(params->referenceList, reference);
+				params->referenceList = g_list_append(params->referenceList, 
+					reference);
 			}
 		}
 	}
@@ -1096,7 +1194,7 @@ qsf_object_commitCB(gpointer key, gpointer value, gpointer data)
 	}
 	if(safe_strcmp(qof_type, QOF_TYPE_INT64) == 0) {
 		errno = 0;
-		cm_i64 = strtoll((char*)xmlNodeGetContent(node), &tail, 0);
+		cm_i64 = strtoll((gchar*)xmlNodeGetContent(node), &tail, 0);
 		if(errno == 0) {
 			i64_setter = (void(*)(QofEntity*, gint64))cm_setter;
 			if(i64_setter != NULL) { i64_setter(qsf_ent, cm_i64); }
@@ -1105,14 +1203,14 @@ qsf_object_commitCB(gpointer key, gpointer value, gpointer data)
 	}
 	if(safe_strcmp(qof_type, QOF_TYPE_DOUBLE) == 0) {
 		errno = 0;
-		cm_double = strtod((char*)xmlNodeGetContent(node), &tail);
+		cm_double = strtod((gchar*)xmlNodeGetContent(node), &tail);
 		if(errno == 0) {
 			double_setter = (void(*)(QofEntity*, double))cm_setter;
 			if(double_setter != NULL) { double_setter(qsf_ent, cm_double); }
 		}
 	}
 	if(safe_strcmp(qof_type, QOF_TYPE_BOOLEAN) == 0){
-		if(0 == safe_strcasecmp((char*)xmlNodeGetContent(node), 
+		if(0 == safe_strcasecmp((gchar*)xmlNodeGetContent(node), 
 				QSF_XML_BOOLEAN_TEST)) {
 			cm_boolean = TRUE;
 		}
@@ -1121,11 +1219,11 @@ qsf_object_commitCB(gpointer key, gpointer value, gpointer data)
 		if(boolean_setter != NULL) { boolean_setter(qsf_ent, cm_boolean); }
 	}
 	if(safe_strcmp(qof_type, QOF_TYPE_KVP) == 0) {
-		cm_type = qsf_to_kvp_helper((char*)xmlGetProp(node, BAD_CAST QSF_OBJECT_VALUE));
+		cm_type = qsf_to_kvp_helper((gchar*)xmlGetProp(node, BAD_CAST QSF_OBJECT_VALUE));
 		if(!cm_type) { return; }
-		cm_value = string_to_kvp_value((char*)xmlNodeGetContent(node), cm_type);
+		cm_value = string_to_kvp_value((gchar*)xmlNodeGetContent(node), cm_type);
 		cm_kvp = (KvpFrame*)cm_param->param_getfcn(qsf_ent, cm_param);
-		cm_kvp = kvp_frame_set_value(cm_kvp, (char*)xmlGetProp(node, 
+		cm_kvp = kvp_frame_set_value(cm_kvp, (gchar*)xmlGetProp(node, 
 			BAD_CAST QSF_OBJECT_KVP), cm_value);
 	}
 	if(safe_strcmp(qof_type, QOF_TYPE_COLLECT) == 0) {
@@ -1137,7 +1235,7 @@ qsf_object_commitCB(gpointer key, gpointer value, gpointer data)
 		qsf_coll = cm_param->param_getfcn(qsf_ent, cm_param);
 		type = qof_collection_get_type(qsf_coll);
 		cm_guid = g_new(GUID, 1);
-		if(TRUE != string_to_guid((char*)xmlNodeGetContent(node), cm_guid))
+		if(TRUE != string_to_guid((gchar*)xmlNodeGetContent(node), cm_guid))
 		{
 			qof_backend_set_error(params->be, ERR_QSF_BAD_OBJ_GUID);
 			PINFO (" string to guid collect failed for %s", xmlNodeGetContent(node));
@@ -1158,9 +1256,9 @@ qsf_object_commitCB(gpointer key, gpointer value, gpointer data)
 		params->referenceList = g_list_append(params->referenceList, reference);
 	}
 	if(safe_strcmp(qof_type, QOF_TYPE_CHAR) == 0) {
-		char_getter = (char (*)(xmlNodePtr))xmlNodeGetContent;
+		char_getter = (gchar (*)(xmlNodePtr))xmlNodeGetContent;
 		cm_char = char_getter(node);
-		char_setter = (void(*)(QofEntity*, char))cm_setter;
+		char_setter = (void(*)(QofEntity*, gchar))cm_setter;
 		if(char_setter != NULL) { char_setter(qsf_ent, cm_char); }
 	}
 }
@@ -1221,12 +1319,6 @@ qsf_provider_init(void)
 {
 	QofBackendProvider *prov;
 
-	#ifdef ENABLE_NLS
-	setlocale (LC_ALL, "");
-	bindtextdomain (GETTEXT_PACKAGE, LOCALE_DIR);
-	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
-	textdomain (GETTEXT_PACKAGE);
-	#endif
 	prov = g_new0 (QofBackendProvider, 1);
 	prov->provider_name = "QSF Backend Version 0.2";
 	prov->access_method = "file";
