@@ -2,7 +2,7 @@
  *            qsf-xml.h
  *
  *  Fri Nov 26 19:29:47 2004
- *  Copyright  2004-2005  Neil Williams  <linux@codehelp.co.uk>
+ *  Copyright  2004,2005,2006  Neil Williams  <linux@codehelp.co.uk>
  *
  ****************************************************************************/
 /*
@@ -14,7 +14,7 @@
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
@@ -31,48 +31,12 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <glib.h>
 #include <regex.h>
 #include <time.h>
-#include <libxml/xmlmemory.h>
-#include <libxml/tree.h>
-#include <libxml/parser.h>
-#include <libxml/xmlschemas.h>
 #include "qof.h"
 
-#if defined(HAVE_GETTEXT)             /* HAVE_GETTEXT */
-
 #include <libintl.h>
-#include <locale.h>
-
-#undef _
-#undef Q_
-
-#ifdef DISABLE_GETTEXT_UNDERSCORE
-#define _(String) (String)
-#define Q_(String) gnc_qualifier_prefix_noop(String)
-#else                                 /* ENABLE_GETTEXT_UNDERSCORE */
-#define _(String) gettext(String)
-#define Q_(String) gnc_qualifier_prefix_gettext(String)
-#endif                                /* End ENABLE_GETTEXT_UNDERSCORE */
-
-#else                                 /* Not HAVE_GETTEXT */
-#if !defined(__USE_GNU_GETTEXT)
-
-#undef _
-#undef Q_
-#define _(String)       (String)
-#define Q_(String) gnc_qualifier_prefix_noop(String)
-#define gettext(String) (String)
-#define ngettext(msgid, msgid_plural, n) (((n)==1) ? \
-                                            (msgid) : (msgid_plural))
-
-#endif                                /* End not__USE_GNU_GETTEXT */
-#endif                                /* End Not HAVE_GETTEXT */
-
-#undef  N_
-#define N_(String) (String)
-
+#define _(String) dgettext (GETTEXT_PACKAGE, String)
 
 typedef enum  {
 	QSF_UNDEF = 0, /**< Initial undefined value. */
@@ -94,7 +58,7 @@ typedef struct qsf_object_set
 {
 	GHashTable *parameters;
 	QofIdType object_type;
-	int object_count;
+	gint object_count;
 }qsf_objects;
 
 #define QSF_QOF_VERSION QOF_OBJECT_VERSION /**< QOF Version check.
@@ -156,6 +120,8 @@ No text content allowed.
 Attributes: e_type Copied directly from the QofObject definition.
 Content: The full QofObject description for the defined QOF object.
 */
+#define MAP_ITERATE_ATTR "foreach" /**< Dictate which object type is the basis
+for iteration in a hierarchical object set. */
 #define MAP_DEFAULT_TAG	"default"  /**< User editable defaults for data not
 available within the available QSF objects.
 
@@ -260,6 +226,8 @@ the calculation where the default is used.
 The value of a calculation is the name of the parameter that will be set by that
 calculation.
 */
+#define MAP_OBJECT_ATTR "object" /**< The object to use to provide the data
+being set using the map.*/
 #define MAP_E_TYPE	"e_type" /**< Validates the objects defined in the map 
 
 The e_type will be used to match incoming QSF objects with the relevant QSF map.
@@ -363,6 +331,35 @@ Both defaults use UTC.
 
 #define QSF_OBJECT_SCHEMA "qsf-object.xsd.xml" /**< Name of the QSF Object Schema. */
 #define QSF_MAP_SCHEMA "qsf-map.xsd.xml" /**< Name of the QSF Map Schema. */
+
+/** \brief Status of various object during mapping.
+
+When handling a map, the incoming QSF objects are not registered with this
+instance of QOF - they originate from another QOF user. Each object in a map
+needs to be defined. If the object is registered, the map is checked to locate
+a calculation that can be used to generate this object. If the object is not
+registered, the incoming QSF is checked to ensure it provides the object data
+for the calculation. If anything goes wrong, QSF_INVALID_OBJECT is used.
+
+Maps can be unidirectional or bidirectional so QOF registration is used to
+determine which calculations should be used and which should be ignored.
+
+All QSF_REGISTERED_OBJECT types need a calculation - if any types remain tagged
+as QSF_REGISTERED_OBJECT when the map validation is complete, the validation
+must fail. The only acceptable end values for QsfStatus are QSF_DEFINED_OBJECT,
+QSF_CALCULATED_OBJECT or QSF_INVALID_OBJECT.
+*/
+typedef enum {
+	QSF_NO_OBJECT = 0,     /**< Init value only. */
+	QSF_DEFINED_OBJECT,    /**< The object is unregistered but defined.
+Objects of this type must exist in the incoming QSF and must
+provide data for the calculation of registered objects. */
+	QSF_REGISTERED_OBJECT, /**< Temporary value. The object is registered
+and defined - a calculation is needed but has not been found, yet. */
+	QSF_CALCULATED_OBJECT, /**< The object is registered, defined and can be calculated. */
+	QSF_INVALID_OBJECT     /**< Oops value. */
+}QsfStatus;
+
 /** \brief QSF Parameters
 
 This struct is a catch-all for all parameters required
@@ -373,41 +370,46 @@ typedef struct qsf_metadata
 {
 	qsf_type file_type;          /**< what type of file is being handled */
 	qsf_objects *object_set;     /**< current object set for qsf_object_list. */
-	int count;                   /**< sequential counter for each object in the book */
+	gint count;                  /**< sequential counter for each object in the book */
 	GList *qsf_object_list;      /**< list of qsf_objects */
 	GSList *qsf_sequence;        /**< Parameter list sorted into QSF order */
 	GList *referenceList;        /**< Table of references, ::QofEntityReference. */
 	GHashTable *qsf_parameter_hash; /**< Hashtable of parameters for each object */
 	GHashTable *qsf_calculate_hash, *qsf_default_hash, *qsf_define_hash;
-	GSList *supported_types;     /**< The list of QOF types currently supported, in QSF order. */
-	xmlDocPtr input_doc;         /**< Pointer to the input xml document(s). */
-	xmlDocPtr output_doc;        /**< Pointer to the output xml document(s). */
-	xmlNodePtr child_node;       /**< The current child_node. */
-	xmlNodePtr convert_node;     /**< Node in the converted object */
-	xmlNodePtr param_node;       /**< Node for parameter data. */
-	xmlNodePtr output_node;      /**< Node in the output document. */
-	xmlNodePtr output_root;      /**< Root node of the output document. */
-	xmlNodePtr book_node;        /**< Node for the book. */
-	xmlNodePtr lister;           /**< Comparison node for map defaults. */
-	xmlNsPtr qsf_ns, map_ns;     /**< Separate namespaces for QSF objects and QSF maps. */
-	const char *qof_type;        /**< Holds details of the QOF_TYPE */
-	QofIdType qof_obj_type;	     /**< current QofObject type (e_type) for the parameters. */
-	QofEntity *qsf_ent;          /**< Current entity in the book. */
-	QofBackend *be;              /**< the current QofBackend for this operation. */
-	gboolean knowntype;          /**< detect references by comparing with known QOF types. */
-	QofParam *qof_param;         /**< used by kvp to handle the frame hash table */
-	QofBook *book;	             /**< the current QofBook.
+	GSList *supported_types;     /**< The list of QOF types currently supported,
+								in QSF order. */
+	xmlDocPtr input_doc;     /**< Pointer to the input xml document(s). */
+	xmlDocPtr output_doc;    /**< Pointer to the output xml document(s). */
+	xmlNodePtr child_node;   /**< The current child_node. */
+	xmlNodePtr convert_node; /**< Node in the converted object */
+	xmlNodePtr param_node;   /**< Node for parameter data. */
+	xmlNodePtr output_node;  /**< Node in the output document. */
+	xmlNodePtr output_root;  /**< Root node of the output document. */
+	xmlNodePtr book_node;    /**< Node for the book. */
+	xmlNodePtr lister;       /**< Comparison node for map defaults. */
+	xmlNsPtr qsf_ns, map_ns; /**< Separate namespaces for QSF objects and QSF maps. */
+	const gchar *qof_type;   /**< Holds details of the QOF_TYPE */
+	QofIdType qof_obj_type;  /**< current QofObject type (e_type) for the parameters. */
+	QofIdType qof_foreach;   /**< How to iterate over hierarchical entities. */
+	gint foreach_limit;      /**< How many iterations are found in the QSF */
+	QofEntity *qsf_ent;      /**< Current entity in the book. */
+	QofBackend *be;          /**< the current QofBackend for this operation. */
+	gboolean knowntype;      /**< detect references by comparing with known QOF types. */
+	QofParam *qof_param;     /**< used by kvp to handle the frame hash table */
+	QofBook *book;           /**< the current QofBook.
 
 		Theoretically, QSF can handle multiple QofBooks - currently limited to 1.
 	*/
-	int boolean_calculation_done; /**< simple trip once this boolean is complete. */
-	char *filepath;               /**< Path to the QSF file. */
+	gint boolean_calculation_done; /**< simple trip once this boolean is complete. */
+	gchar *filepath;              /**< Path to the QSF file. */
+	gchar *map_path;              /**< Path to best match map, if any. */
 	gchar* full_kvp_path;         /**< Full path for each KvpValue written out. */
-	gint use_gz_level;            /**< Default compression level. */
+	gint64 use_gz_level;          /**< Default compression level. */
 	GList *map_files;             /**< List of selected map files for this session.
 
 	Defaults to the pre-installed QSF maps, currently: pilot-qsf-GnuCashInvoice.xml
 	*/
+	const gchar *encoding;        /**< Backend encoding option - defaults to UTF-8. */ 
 }qsf_param;
 
 /** \brief Validation metadata
@@ -418,12 +420,21 @@ This is used to determine which backend should load the data.
 typedef struct qsf_validates
 {
 	QofBackendError error_state;
-	const char *object_path;
-	const char *map_path;
-	GHashTable *validation_table;
-	int valid_object_count;
-	int map_calculated_count;
-	int qof_registered_count;
+	const gchar *object_path;
+	const gchar *map_path;
+	GHashTable *object_table;  /**< Names of all incoming objects (from the
+								object_type) and status (QOF registration). */
+	GHashTable *map_table;     /**< Names of all defined objects (from the
+								define tag) and status (presence of a calculation.)*/
+	/* Need to match object names, not just counts. */
+	gint valid_object_count;   /**< Number of unique incoming objects as 
+								defined in ::QSF_OBJECT_TAG tags in the QSF.*/
+	gint map_calculated_count; /**< Number of objects that can be calculated
+								by this map. ::MAP_OBJECT_TAG. */
+	gint qof_registered_count; /**< Number of objects (in either the QSF 
+								or the map) that are registered with QofObject. */
+	gint incoming_count;       /**< Number of unique objects in the incoming
+								QSF file. Used to ensure all incoming objects are used.*/
 }qsf_validator;
 
 /** \brief shorthand function
@@ -431,32 +442,32 @@ typedef struct qsf_validates
 This may look repetitive but each one is used separately
 as well as in a block.
 */
-int
-qsf_compare_tag_strings(const xmlChar *node_name, char *tag_name);
+gint
+qsf_compare_tag_strings(const xmlChar *node_name, gchar *tag_name);
 
 /** \brief shorthand function
 
 This may look repetitive but each one is used separately
 as well as in a block.
 */
-int
-qsf_strings_equal(const xmlChar *node_name, char *tag_name);
+gint
+qsf_strings_equal(const xmlChar *node_name, gchar *tag_name);
 
 /** \brief shorthand function
 
 This may look repetitive but each one is used separately
 as well as in a block.
 */
-int
-qsf_is_element(xmlNodePtr a, xmlNsPtr ns, char *c);
+gint
+qsf_is_element(xmlNodePtr a, xmlNsPtr ns, gchar *c);
 
 /** \brief shorthand function
 
 This may look repetitive but each one is used separately
 as well as in a block.
 */
-int
-qsf_check_tag(qsf_param *params, char *qof_type);
+gint
+qsf_check_tag(qsf_param *params, gchar *qof_type);
 
 /** \brief Checks all incoming objects for QOF registration.
 
@@ -479,7 +490,7 @@ Incorrect validation will result in output to the terminal window.
 @return TRUE if the doc validates against the assigned schema, otherwise FALSE.
 */
 gboolean
-qsf_is_valid(const char *schema_dir, const char* schema_filename, xmlDocPtr doc);
+qsf_is_valid(const gchar *schema_dir, const gchar* schema_filename, xmlDocPtr doc);
 
 /** \brief Prepare the default list of maps.
 
@@ -562,7 +573,7 @@ file are defined in the QSF map.
 @return TRUE if the file validates and a QSF map can be found,
 otherwise FALSE.
 */
-gboolean is_qsf_object(const char *path);
+gboolean is_qsf_object(const gchar *path);
 /** \brief Validate a QSF file and determine type.
 
 @param	params	Pointer to qsf_param context
@@ -592,7 +603,7 @@ for a QSF map.
 @return TRUE if the file validates and all objects pass,
 otherwise FALSE.
 */
-gboolean is_our_qsf_object(const char *path);
+gboolean is_our_qsf_object(const gchar *path);
 /** \brief Validate a QSF map file.
 
 @param	params	Pointer to qsf_param context
@@ -621,7 +632,7 @@ user data but are used to import QSF object files from other applications.
 
 @return TRUE if the map validates, otherwise FALSE.
 */
-gboolean is_qsf_map(const char *path);
+gboolean is_qsf_map(const gchar *path);
 /** \brief Validate a QSF file and a selected QSF map
 
 @param	map_path	Absolute or relative path to the selected QSF map file
@@ -637,7 +648,7 @@ This backend twin also sets QofBackendError codes.
 @return TRUE if the file validates and the supplied QSF map is usable,
 otherwise FALSE.
 */
-gboolean is_qsf_object_with_map_be(char *map_path, qsf_param *params);
+gboolean is_qsf_object_with_map_be(gchar *map_path, qsf_param *params);
 /** \brief Validate a QSF file and a selected QSF map
 
 @param	map_path	Absolute or relative path to the selected QSF map file
@@ -651,7 +662,7 @@ file are defined in the QSF map.
 @return TRUE if the file validates and the supplied QSF map is usable,
 otherwise FALSE.
 */
-gboolean is_qsf_object_with_map(const char *path, char *map_file);
+gboolean is_qsf_object_with_map(const gchar *path, gchar *map_file);
 
 /** @} */
 
@@ -676,7 +687,7 @@ value.
 @return KvpValue* or NULL on failure.
 */
 KvpValue*
-string_to_kvp_value(const char *content, KvpValueType type);
+string_to_kvp_value(const gchar *content, KvpValueType type);
 
 /** Validate the children of the parent node.
 
