@@ -28,9 +28,14 @@
 #define __EXTENSIONS__
 
 #include "config.h"
+#include <glib.h>
 /* to be renamed qofdate.c */
 #include <ctype.h>
 
+#ifdef HAVE_LANGINFO_H
+#define HAVE_LANGINFO_D_FMT 1
+#endif
+
 #ifdef HAVE_LANGINFO_D_FMT
 #include <langinfo.h>
 #endif
@@ -934,10 +939,10 @@ char dateSeparator (void)
         return locale_separator;
       else
       { /* Make a guess */
-        char string[256];
+        unsigned char string[256];
         struct tm *tm;
         time_t secs;
-        char *s;
+        unsigned char *s;
 
         secs = time(NULL);
         tm = localtime(&secs);
@@ -1055,7 +1060,7 @@ gnc_iso8601_to_timespec_gmt(const char *str)
 
     str +=3;
     if ('.' == *str) str++;
-    if (isdigit (*str) && isdigit (*(str+1)))
+    if (isdigit ((unsigned char)*str) && isdigit ((unsigned char)*(str+1)))
     {
       int cyn;
       /* copy sign from hour part */
@@ -1425,7 +1430,11 @@ qof_date_add_days(Timespec *ts, gint days)
 
 	g_return_val_if_fail(ts, FALSE);
 	tt = timespecToTime_t(*ts);
+#ifdef HAVE_GMTIME_R
 	tm = *gmtime_r(&tt, &tm);
+#else
+	tm = *gmtime(&tt);
+#endif
 	tm.tm_mday += days;
 	/* let mktime normalise the months and year
 	because we aren't tracking last_day_of_month */
@@ -1445,7 +1454,11 @@ qof_date_add_months(Timespec *ts, gint months, gboolean track_last_day)
 
 	g_return_val_if_fail(ts, FALSE);
 	tt = timespecToTime_t(*ts);
+#ifdef HAVE_GMTIME_R
 	tm = *gmtime_r(&tt, &tm);
+#else
+	tm = *gmtime(&tt);
+#endif
 	was_last_day = date_is_last_mday(&tm);
 	tm.tm_mon += months;
 	while (tm.tm_mon > 11) {
